"""
This file is a mockup for Read the Docs buil.
"""

# win32con
def GetMOduleHandle(*args, **kwargs):
    pass

CW_USEDEFAULT = -2147483648
IMAGE_ICON = 1
LR_DEFAULTSIZE = 64
LR_LOADFROMFILE = 16
WM_DESTROY = 2
WM_USER = 1024
WS_OVERLAPPED = 0
WS_SYSMENU = 524288

# win32gui
def CreateWindow(*args, **kwargs):
    pass

def DefWindowProc(*args, **kwargs):
    pass

def DestroyWindow(*args, **kwargs):
    pass

def LoadImage(*args, **kwargs):
    pass

NIF_ICON = 2
NIF_INFO = 16
NIF_MESSAGE = 1
NIF_TIP = 4
NIIF_INFO = 1
NIIF_ERROR = 3
NIIF_NOSOUND = 16
NIIF_WARNING = 2
NIM_ADD = 0
NIM_DELETE = 2
NIM_MODIFY = 1

def PostQuitMessage(*args, **kwargs):
    pass
    
def PumpWaitingMessages(*args, **kwargs):
    pass

def RegisterClass(*args, **kwargs):
    pass

def Shell_NotifyIcon(*args, **kwargs):
    pass

def UnregisterClass(*args, **kwargs):
    pass

def UpdateWindow(*args, **kwargs):
    pass

def WNDCLASS(*args, **kwargs):
    pass